﻿if (! _global._RAY07_AS) 
{
	_global._RAY07_AS = true;

Test_Intersect = function(ray_direction, origin, sphere, light, fall_off,Ambient,KDiffuse,KSpecular,Sphere_list_Length,Sphere_List){
	//test for intersect with sphere
		cpx = origin.x-sphere.x;
		cpy = origin.y-sphere.y;
		cpz = origin.z-sphere.z;
		
		b = 2 * ( ray_direction.x*cpx + ray_direction.y*cpy + ray_direction.z*cpz);
		c = cpx*cpx + cpy*cpy + cpz*cpz - (sphere.rad*sphere.rad);
		bc = Math.sqrt( (b*b) - (4*c) );
		
		if(bc >= 0){
			
		//get intersection point
			w1 = (-b + bc)*0.5;
			w2 = (-b - bc)*0.5;
		
			if(w1<w2)
			{
				w = w1;
				wb = w2;
			} else { 
				w = w2;
				wb = w1;
			}

			if(w<0){
				w = wb;
			}
			
			x = origin.x + ray_direction.x*w;
			y = origin.y + ray_direction.y*w;
			z = origin.z + ray_direction.z*w;
		
		//attenuation
		distance = Math.sqrt(x*x + y*y + z*z);
		attenuation_distance = fall_off/distance;
		attenuation = 1 - (1/(attenuation_distance*attenuation_distance));

		
		// CASTING A SHADOW RAY
			
			dn = Math.sqrt(light.x*light.x + light.y*light.y + light.z*light.z);
			L = new Vector3D(light.x/dn, light.y/dn, light.z/dn);
			shadow_spot = 0;
			
			for(SV=0;SV<Sphere_list_Length;SV++){
				ssphere = Sphere_List[SV];
	
				if(ssphere.no != sphere.no){
				Scpx = x-ssphere.x;
				Scpy = y-ssphere.y;
				Scpz = z-ssphere.z;
			
				sb = 2 * ( L.x*Scpx + L.y*Scpy + L.z*Scpz);
				sc = Scpx*Scpx + Scpy*Scpy + Scpz*Scpz - (ssphere.rad*ssphere.rad);
				sbc = Math.sqrt( (sb*sb) - (4*sc) );
					if(sbc > 0){
						sw1 = (-sb + sbc)*0.5;
						sw2 = (-sb - sbc)*0.5;
					
						if(sw1<sw2)
						{
							sw = sw1;
							swb = sw2;
						} else { 
							sw = sw2;
							swb = sw1;
						}
						
						if(sw<0){
							sw = swb;
						}
			
						if(sw>0){
							shadow_spot = 1;
						}
					}
				}
			}

		//CALCULATIONS - THE REAL STUFF
		
		//Diffuse = N.L
			N = new Vector3D((x-sphere.x)/sphere.rad, (y-sphere.y)/sphere.rad, (z-sphere.z)/sphere.rad);

			Df = (N.x*L.x) + (N.y*L.y) + (N.z*L.z);
			Diffuse = Df*KDiffuse;
			if(Diffuse<0 || shadow_spot == 1){
				Diffuse = 0;
			}

		//(Specular = (R.V) ^ n              //R = 2.0*(N.L)*N - L
			sv = ray_direction;
			
			sr = new Vector3D((2*Df*N.x) - L.x, (2*Df*N.y) - L.y, (2*Df*N.z) - L.z);
			sdn = Math.sqrt((sr.x*sr.x) + (sr.y*sr.y) + (sr.z*sr.z));
			sr = new Vector3D(-sr.x/sdn, -sr.y/sdn, -sr.z/sdn);
			
			Sp = (sv.x*sr.x) + (sv.y*sr.y) + (sv.z*sr.z);
			Specular = Sp*KSpecular;
			Specular = Math.pow(Specular,9);
			if(Specular<0 || shadow_spot == 1 || Diffuse ==0){
				Specular = 0;
			}
		
		//DO FINAL LIGHTING CALCULATIONS
	
		r = (Ambient*((light.red + sphere.red)*0.5)) + attenuation*light.red*((sphere.red*Diffuse/255) + Specular);
		g = (Ambient*((light.green + sphere.green)*0.5)) + attenuation*light.green*((sphere.green*Diffuse/255) + Specular);
		b = (Ambient*((light.blue + sphere.blue)*0.5)) + attenuation*light.blue*((sphere.blue*Diffuse/255) + Specular);

		if(r>255){r = 255;}
		if(g>255){g = 255;}
		if(b>255){b = 255;}
		
		colour = new Colour(r, g, b);
		
		} else {
			
		colour = 'NONE';
		dist = 99999999;
		}
		
		return_value = new Array(colour, distance);
		return(return_value);
}

// FUNCTIONS AND CLASSES

Colour = function(colour_red, colour_green, colour_blue)
{
	this.red = colour_red;
	this.green = colour_green;
	this.blue = colour_blue;
}

Sphere = function (number, cx, cy, cz, radius, reflection, colour_red, colour_green, colour_blue)
{
	this.x = cx;
	this.y = cy;
	this.z = cz;
	this.rad = radius;
	this.ref = reflection;
	this.red = colour_red;
	this.green = colour_green;
	this.blue = colour_blue;
	this.no = number
}

Light = function (cx, cy, cz, radius, colour_red, colour_green, colour_blue)
{
	this.x = cx;
	this.y = cy;
	this.z = cz;
	this.rad = radius;
	this.red = colour_red;
	this.green = colour_green;
	this.blue = colour_blue;
}

Vector3D = function (vx, vy, vz)
{
	this.x = vx;
	this.y = vy;
	this.z = vz;
}

Dot = function(a,b){
	c = (a.x*b.x) + (a.y*b.y) + (a.z*b.z);
	return (c);
}

Normalize = function(n){
	
	dn = Math.sqrt(n.x*n.x + n.y*n.y + n.z*n.z);
	n.x = n.x/dn;
	n.y = n.y/dn;
	n.z = n.z/dn;
	
	return(n);
}

}